@@ -1,8 +1,11 @@
 import collections.abc
 import inspect
+from contextlib import AsyncExitStack, asynccontextmanager, contextmanager
 from functools import lru_cache
 from typing import Any, Sequence, Tuple, Type, get_args, get_origin
 
+from flexdi.errors import UntypedError
+
 
 @lru_cache(maxsize=None)
 def get_cached_class_args(clazz: Type[Any]) -> Tuple[Type[Any], Sequence[Type[Any]]]:
@@ -33,7 +36,7 @@ def determine_return_type(obj: Any) -> Any:
     return_type = signature.return_annotation
 
     if return_type is signature.empty:
-        return obj
+        raise UntypedError(f"Could not determine return type for {obj}")
 
     clazz_origin, clazz_args = get_cached_class_args(return_type)
 
@@ -41,5 +44,25 @@ def determine_return_type(obj: Any) -> Any:
         return clazz_args[0]
     elif clazz_origin is collections.abc.AsyncIterator:
         return clazz_args[0]
+    elif clazz_origin is collections.abc.Generator:
+        return clazz_args[0]
+    elif clazz_origin is collections.abc.AsyncGenerator:
+        return clazz_args[0]
 
     return return_type
+
+
+async def invoke_callable(stack: AsyncExitStack, func: Any, args: Any) -> Any:
+    if inspect.isasyncgenfunction(func):
+        async_context_func = asynccontextmanager(func)
+        return await stack.enter_async_context(async_context_func(**args))
+
+    elif inspect.isgeneratorfunction(func):
+        sync_context_func = contextmanager(func)
+        return stack.enter_context(sync_context_func(**args))
+
+    elif inspect.iscoroutinefunction(func):
+        return await func(**args)
+
+    else:
+        return func(**args)
